ColorGradingEditorItemsFrame = {
	CONTROLS = {
		"title",
		"itemList"
	}
}

local ColorGradingEditorItemsFrame_mt = Class(ColorGradingEditorItemsFrame, TabbedMenuFrameElement)

local function NO_CALLBACK()
end

function ColorGradingEditorItemsFrame.new(target, subclass_mt)
	local self = ColorGradingEditorItemsFrame:superClass().new(target, subclass_mt or ColorGradingEditorItemsFrame_mt)

	self:registerControls(ColorGradingEditorItemsFrame.CONTROLS)

	self.dayName = "day"
	self.sectionName = "saturation"

	return self
end

function ColorGradingEditorItemsFrame:onFrameClose()
	self.itemList:scrollTo(0, true)
	ColorGradingEditorItemsFrame:superClass().onFrameClose(self)
end

function ColorGradingEditorItemsFrame:initialize(dayName, sectionName, delegate)
	self.title:setText(string.format("%s - %s", g_i18n:getText("ui_cge_" .. dayName), g_i18n:getText("ui_cge_" .. sectionName)))
	self.delegate = delegate
	self.dayName = dayName
	self.sectionName = sectionName
end

function ColorGradingEditorItemsFrame:getGroupByIndex(sectionIndex)
	local preset = self.delegate:getCurrentPreset()
	local sections = preset.colorGrading[self.dayName]
	local groups = sections[self.sectionName]

	for i, groupName in pairs(self.delegate.groupNames) do
		if sectionIndex == i and groups[groupName] ~= nil then
			return groups[groupName], groupName
		end
	end

	if sectionIndex == 5 then
		if self.sectionName == "shadows" then
			return {sections.shadowsMaxLuminance}, "shadowsMaxLuminance"
		elseif self.sectionName == "highlights" then
			return {sections.highlightsMinLuminance}, "highlightsMinLuminance"
		end
	end

	return {}, ""
end

function ColorGradingEditorItemsFrame:onGuiSetupFinished()
	ColorGradingEditorItemsFrame:superClass().onGuiSetupFinished(self)

	ColorGradingEditorUtil.overwrittenFunction(self.itemList, "shouldFocusChange", self, "itemList_shouldFocusChange")
	ColorGradingEditorUtil.overwrittenFunction(self.itemList, "playSample", self, "itemList_playSample")
end

function ColorGradingEditorItemsFrame:onMapLoad()
	self.itemList:reloadData()
end

function ColorGradingEditorItemsFrame:getNumberOfSections()
	if self.sectionName == "shadows" or self.sectionName == "highlights" then
		return 5
	end

	return 4
end

function ColorGradingEditorItemsFrame:getTitleForSectionHeader(list, sectionIndex)
	local _, groupName = self:getGroupByIndex(sectionIndex)

	return g_i18n:getText("ui_cge_" .. groupName)
end

function ColorGradingEditorItemsFrame:getNumberOfItemsInSection(list, sectionIndex)
	local values, _ = self:getGroupByIndex(sectionIndex)

	return #values
end

function ColorGradingEditorItemsFrame:populateCellForItemInSection(list, sectionIndex, index, cell)
	local values, _ = self:getGroupByIndex(sectionIndex)
	local value = values[index]
	local slider = cell:getAttribute("slider")
	local profile = "cgeListItemSlider_" .. tostring(index)

	if #values == 1 then
		profile = "cgeListItemSlider_4"
	end

	local lastFilename = slider.overlay.filename

	slider:applyProfile(profile)

	if lastFilename ~= slider.overlay.filename then
		GuiOverlay.deleteOverlay(slider.overlay)
		GuiOverlay.createOverlay(slider.overlay)
	end

	slider.textElement = cell:getAttribute("text")
	self:setSliderValue(slider, value)
end

function ColorGradingEditorItemsFrame:setSliderValue(slider, value)
	local onChangedCallbackBackup = slider.onChangedCallback

	slider.onChangedCallback = NO_CALLBACK
	slider:setValue(value)
	slider.onChangedCallback = onChangedCallbackBackup
end

function ColorGradingEditorItemsFrame:onSliderValueChange(slider, value)
	local selectedItem = self.itemList:getSelectedElement()
	local item = slider.parent

	if selectedItem ~= item then
		self:setSoundSuppressed(true)
		self.itemList:setSelectedItem(item.sectionIndex, item.indexInSection)
		self:setSoundSuppressed(false)
	end

	self.delegate:onSlidersUpdate(self, slider, value)
end

function ColorGradingEditorItemsFrame:onSliderMiddleClick(slider)
	local item = slider.parent

	local sectionIndex = item.sectionIndex
	local index = item.indexInSection

	local preset = self.delegate:getDefaultPreset()
	local sections = preset.colorGrading[self.dayName]
	local groups = sections[self.sectionName]

	local value = nil

	for i, groupName in pairs(self.delegate.groupNames) do
		if sectionIndex == i and groups[groupName] ~= nil then
			value = groups[groupName][index]
			break
		end
	end

	if sectionIndex == 5 and index == 1 then
		if self.sectionName == "shadows" then
			value = sections.shadowsMaxLuminance
		elseif self.sectionName == "highlights" then
			value = sections.highlightsMinLuminance
		end
	end

	if value ~= nil then
		slider:setValue(value)
	end
end

function ColorGradingEditorItemsFrame:itemList_shouldFocusChange(list, superFunc, direction)
	local selectedIndex = list.selectedIndex
	local sectionIndex = list.selectedSectionIndex

	local retValue = superFunc(list, direction)

	local selectedElement = list:getSelectedElement()

	if selectedElement ~= nil then
		local slider = selectedElement:getDescendantByName("slider")

		if slider ~= nil then
			slider:shouldFocusChange(direction)
		end
	end

	if list.totalItemCount > 0 then
		local dir1 = FocusManager.TOP
		local dir2 = FocusManager.BOTTOM

		if direction == dir1 or direction == dir2 then
			local lastSectionIndex = #list.sections
			local lastIndex = list.sections[lastSectionIndex].numItems

			if direction == dir1 then
				if sectionIndex == 1 and selectedIndex == 1 then
					list:setSelectedItem(lastSectionIndex, lastIndex, true, 0)

					return false
				end
			else
				if sectionIndex == lastSectionIndex and selectedIndex == lastIndex then
					list:setSelectedItem(1, 1, true, 0)

					return false
				end
			end
		end
	end

	return retValue
end

function ColorGradingEditorItemsFrame:itemList_playSample(list, superFunc, sampleName)
	if not list:getSoundSuppressed() then
		superFunc(list, sampleName)
	end
end