ColorGradingEditorScreen = {
	CONTROLS = {
		"background",
		"menuButton",
		"pageDayGlobal",
		"pageDayShadows",
		"pageDayMidtones",
		"pageDayHighlights",
		"pageNightGlobal",
		"pageNightShadows",
		"pageNightMidtones",
		"pageNightHighlights",
		"pageSettings",
		"multiPreset",
		"checkHelp",
		"multiBg"
	}
}

local ColorGradingEditorScreen_mt = Class(ColorGradingEditorScreen, TabbedMenuWithDetails)

function ColorGradingEditorScreen.new(target, custom_mt, messageCenter, l10n, inputManager, colorGradingEditor, baseDirectory, settingsDirectory)
	local self = TabbedMenuWithDetails.new(target, custom_mt or ColorGradingEditorScreen_mt, messageCenter, l10n, inputManager)

	self:registerControls(ColorGradingEditorScreen.CONTROLS)

	self.colorGradingEditor = colorGradingEditor

	self.showHelp = true
	self.bgState = 1

	self.settingsXMLFilename = settingsDirectory .. "settings.xml"

	self.settings = {
		changed = false,
		isCustomPreset = false,
		presetName = "",
		showHelp = true,
		bg = 1
	}

	self:loadSettingsFromXMLFile()

	self.baseDirectory = baseDirectory
	self.settingsDirectory = settingsDirectory
	self.presetsDirectory = settingsDirectory .. "presets"

	createFolder(self.settingsDirectory)
	createFolder(self.presetsDirectory)

	self.defaultMenuButtonInfo = {}
	self.pages = {}

	self.blurApplied = false

	self.sectionNames = {
		"global",
		"shadows",
		"midtones",
		"highlights"
	}

	self.groupNames = {
		"saturation",
		"contrast",
		"gamma",
		"gain"
	}

	self.minValue = 0
	self.maxValue = 2

	self.maxPresets = 30

	self.presets = {}
	self.currentPresetId = 1

	self.defaultColorGrading = "data/maps/default_colorGrading.xml"

	self.tempDay = self.settingsDirectory .. "colorGradingDay.xml"
	self.tempNight = self.settingsDirectory .. "colorGradingNight.xml"

	self.saveTimer = 0
	self.saveDelay = 200
	self.saveTimerEnabled = false

	self.updateTimer = 0
	self.updateDelay = 200
	self.updateTimerEnabled = false

	self.sliderSample = GuiSoundPlayer.SOUND_SAMPLES.HOVER

	self.showCurrentGrading = false
	addConsoleCommand("cgeShowCurrentGrading", "", "consoleCommandShowCurrentGrading", self)

	return self
end

function ColorGradingEditorScreen:onGuiSetupFinished()
	ColorGradingEditorScreen:superClass().onGuiSetupFinished(self)

	self:initializePages()
	self:setupMenuPages()

	local function getCanChangeState(...)
		return not self:isInputDisabled()
	end

	self.multiPreset.getCanChangeState = getCanChangeState
	self.checkHelp.getCanChangeState = getCanChangeState
	self.multiBg.getCanChangeState = getCanChangeState
end

function ColorGradingEditorScreen:onMapLoad()
	g_currentMission:registerObjectToCallOnMissionStart(self)

	local baseLighting = g_currentMission.environment.baseLighting

	self.presets = {
		{
			name = "",
			isDefault = true,
			colorGrading = self:loadColorGradingFromFiles(baseLighting.colorGradingDay, baseLighting.colorGradingNight)
		},
		{
			name = "",
			isCustom = true,
			colorGrading = self:loadColorGradingFromFiles(self.tempDay, self.tempNight)
		}
	}

	self:loadUserPresets(self.presets)

	for _, page in pairs(self.pages) do
		page:onMapLoad()
	end

	ColorGradingEditorUtil.overwrittenFunction(Lighting, "update", self, "lighting_update")
end

function ColorGradingEditorScreen:onMissionStarted()
	local _, presetId = nil, nil

	if self:getSetting("isCustomPreset") then
		_, presetId = self:getCustomPreset()
	else
		_, presetId = self:getPresetByName(self:getSetting("presetName"))
	end

	if presetId ~= nil then
		self.multiPreset:setState(presetId, true)
	end

	local bgId = self:getSetting("bg")

	if bgId ~= nil then
		self.multiBg:setState(bgId, true)
	end

	self.checkHelp:setState(self:getSetting("showHelp") and CheckedOptionElement.STATE_CHECKED or CheckedOptionElement.STATE_UNCHECKED, true)
end

function ColorGradingEditorScreen:loadUserPresets(presets)
	local presetsDir = self.presetsDirectory
	local files = Files.new(presetsDir)

	for _, v in pairs(files.files) do
		if v.isDirectory then
			local path = presetsDir .. "/" .. v.filename
			local dayFilename = path .. "/colorGradingDay.xml"
			local nightFilename = path .. "/colorGradingNight.xml"

			if fileExists(dayFilename) and fileExists(nightFilename) then
				table.insert(presets, {
					name = v.filename,
					isUser = true,
					colorGrading = self:loadColorGradingFromFiles(dayFilename, nightFilename)
				})
			end
		end
	end

	self:updatePresetTexts()
end

function ColorGradingEditorScreen:loadColorGradingFromFiles(dayFilename, nightFilename)
	return {
		day = self:loadXMLFile(dayFilename),
		night = self:loadXMLFile(nightFilename)
	}
end

function ColorGradingEditorScreen:loadXMLFile(filename)
	local colorGrading = {
		filename = filename
	}

	if not fileExists(filename) then
		filename = self.defaultColorGrading
	end

	local xmlFile = XMLFile.loadIfExists("colorGradingXML", filename)

	if xmlFile ~= nil then
		local baseKey = "colorGrading"

		colorGrading.shadowsMaxLuminance = xmlFile:getFloat(baseKey .. "#shadowsMaxLuminance", 0.25)
		colorGrading.highlightsMinLuminance = xmlFile:getFloat(baseKey .. "#highlightsMinLuminance", 0.35)

		for _, sectionName in pairs(self.sectionNames) do
			local sectionKey = baseKey .. "." .. sectionName
			local sectionTable = {}

			for _, groupName in pairs(self.groupNames) do
				local key = sectionKey .. "." .. groupName
				local value = xmlFile:getVector(key .. "#rgb", {1, 1, 1}, 3)

				value[4] = xmlFile:getFloat(key .. "#scale", 1)
				sectionTable[groupName] = value
			end

			colorGrading[sectionName] = sectionTable
		end
	end

	return colorGrading
end

function ColorGradingEditorScreen:saveTempColorGradingFiles()
	local preset = self:getCustomPreset()

	if self:saveXMLFile(preset.colorGrading.day) and self:saveXMLFile(preset.colorGrading.night) then
		self.updateTimer = self.updateDelay
		self.updateTimerEnabled = true
	end
end

function ColorGradingEditorScreen:saveXMLFile(colorGrading)
	local xmlFile = XMLFile.create("colorGradingXML", colorGrading.filename, "colorGrading")
	local success = false

	if xmlFile ~= nil then
		local baseKey = "colorGrading"

		xmlFile:setFloat(baseKey .. "#shadowsMaxLuminance", self:clamp(colorGrading.shadowsMaxLuminance))
		xmlFile:setFloat(baseKey .. "#highlightsMinLuminance", self:clamp(colorGrading.highlightsMinLuminance))

		for _, sectionName in pairs(self.sectionNames) do
			local sectionKey = baseKey .. "." .. sectionName

			for _, groupName in pairs(self.groupNames) do
				local key = sectionKey .. "." .. groupName
				local value = colorGrading[sectionName][groupName]

				xmlFile:setVector(key .. "#rgb", {self:clamp(value[1]), self:clamp(value[2]), self:clamp(value[3])})
				xmlFile:setFloat(key .. "#scale", self:clamp(value[4]))
			end
		end

		success = xmlFile:save()
		xmlFile:delete()
	end

	return success
end

function ColorGradingEditorScreen:initializePages()
	self.pageDayGlobal:initialize("day", "global", self)
	self.pageDayShadows:initialize("day", "shadows", self)
	self.pageDayMidtones:initialize("day", "midtones", self)
	self.pageDayHighlights:initialize("day", "highlights", self)
	self.pageNightGlobal:initialize("night", "global", self)
	self.pageNightShadows:initialize("night", "shadows", self)
	self.pageNightMidtones:initialize("night", "midtones", self)
	self.pageNightHighlights:initialize("night", "highlights", self)
	self.pageSettings:initialize(self)

	self.pages = {
		self.pageDayGlobal,
		self.pageDayShadows,
		self.pageDayMidtones,
		self.pageDayHighlights,
		self.pageNightGlobal,
		self.pageNightShadows,
		self.pageNightMidtones,
		self.pageNightHighlights
	}
end

function ColorGradingEditorScreen:setupMenuPages()
	local function rootPagePredicate()
		return not self:getIsDetailMode()
	end

	local iconsFilename = self.colorGradingEditor.uiFilename

	local orderedDefaultPages = {
		{
			self.pageDayGlobal,
			rootPagePredicate,
			{0, 0, 65, 65},
			iconsFilename
		},
		{
			self.pageDayShadows,
			rootPagePredicate,
			{65, 0, 65, 65},
			iconsFilename
		},
		{
			self.pageDayMidtones,
			rootPagePredicate,
			{130, 0, 65, 65},
			iconsFilename
		},
		{
			self.pageDayHighlights,
			rootPagePredicate,
			{195, 0, 65, 65},
			iconsFilename
		},
		{
			self.pageNightGlobal,
			rootPagePredicate,
			{260, 0, 65, 65},
			iconsFilename
		},
		{
			self.pageNightShadows,
			rootPagePredicate,
			{325, 0, 65, 65},
			iconsFilename
		},
		{
			self.pageNightMidtones,
			rootPagePredicate,
			{390, 0, 65, 65},
			iconsFilename
		},
		{
			self.pageNightHighlights,
			rootPagePredicate,
			{455, 0, 65, 65},
			iconsFilename
		},
		{
			self.pageSettings,
			rootPagePredicate,
			{780, 65, 65, 65},
			g_iconsUIFilename
		}
	}

	for i, pageDef in ipairs(orderedDefaultPages) do
		local page, predicate, iconUVs, imageFilename = unpack(pageDef)

		self:registerPage(page, i, predicate)

		local normalizedUVs = GuiUtils.getUVs(iconUVs)

		self:addPageTab(page, imageFilename, normalizedUVs)
	end
end

function ColorGradingEditorScreen:setupMenuButtonInfo()
	ColorGradingEditorScreen:superClass().setupMenuButtonInfo(self)

	local onButtonSaveFunction = self:makeSelfCallback(self.onButtonSave)
	local onButtonBackFunction = self.clickBackCallback

	self.defaultMenuButtonInfo = {
		{
			inputAction = InputAction.MENU_ACCEPT,
			text = self.l10n:getText("button_save"),
			callback = onButtonSaveFunction
		},
		{
			inputAction = InputAction.MENU_BACK,
			text = self.l10n:getText("button_back"),
			callback = onButtonBackFunction
		}
	}

	self.defaultMenuButtonInfoByActions[InputAction.MENU_ACCEPT] = self.defaultMenuButtonInfo[1]
	self.defaultMenuButtonInfoByActions[InputAction.MENU_BACK] = self.defaultMenuButtonInfo[2]

	self.defaultButtonActionCallbacks = {
		[InputAction.MENU_ACCEPT] = onButtonSaveFunction,
		[InputAction.MENU_BACK] = onButtonBackFunction
	}

	self:assignMenuButtonInfo(self.defaultMenuButtonInfo)
end


function ColorGradingEditorScreen:onOpen()
	ColorGradingEditorScreen:superClass().onOpen(self)

	self:disableInputForDuration(250)
	self:setBlurApplied(true)
end

function ColorGradingEditorScreen:onClose()
	if self.saveTimerEnabled then
		self.saveTimer = 0
		self.saveTimerEnabled = false
		self:saveTempColorGradingFiles()
	end

	if self.updateTimerEnabled then
		self.updateTimer = 0
		self.updateTimerEnabled = false
		self:updateColorGrading()
	end

	self:setBlurApplied(false)
	self:saveSettings()

	ColorGradingEditorScreen:superClass().onClose(self)
end

function ColorGradingEditorScreen:update(dt)
	ColorGradingEditorScreen:superClass().update(self, dt)

	if self.saveTimerEnabled then
		self.saveTimer = math.max(self.saveTimer - dt, 0)

		if self.saveTimer == 0 then
			self.saveTimerEnabled = false
			self:saveTempColorGradingFiles()
		end
	end

	if self.updateTimerEnabled then
		self.updateTimer = math.max(self.updateTimer - dt, 0)

		if self.updateTimer == 0 then
			self.updateTimerEnabled = false
			self:updateColorGrading()
		end
	end

	if #g_gui.dialogs > 0 then
		if self.blurApplied then
			self:setBlurApplied(false)
		end
	elseif not self.blurApplied then
		self:setBlurApplied(true)
	end
end

function ColorGradingEditorScreen:onClickPresetChange(state, element, isLeftButtonEvent)
	self.currentPresetId = state

	self:updateSlidersValues()
	self:updateColorGrading()
	self:updateSaveButtonText()

	local agsMultiColorGrading = self.colorGradingEditor.agsMultiColorGrading

	if agsMultiColorGrading ~= nil then
		if element == self.multiPreset then
			agsMultiColorGrading:setState(state)
		elseif element == agsMultiColorGrading then
			self.multiPreset:setState(state)
		end
	end
end

function ColorGradingEditorScreen:onClickShowHelp(state)
	local showHelp = state == CheckedOptionElement.STATE_CHECKED

	g_inputBinding:setActionEventTextVisibility(self.colorGradingEditor.eventId, showHelp)
	self.showHelp = showHelp
end

function ColorGradingEditorScreen:onClickBgChange(state, element, isLeftButtonEvent)
	self.bgState = state

	local pro = "100"

	if state == 2 then
		pro = "80"
	elseif state == 3 then
		pro = "60"
	elseif state == 4 then
		pro = "40"
	elseif state == 5 then
		pro = "20"
	elseif state == 6 then
		pro = "0"
	end

	self.background:applyProfile("cgeFullInGameBackground_" .. pro)
end

function ColorGradingEditorScreen:onButtonSave()
	local preset = self:getCurrentPreset()

	if preset.isUser then
		g_gui:showYesNoDialog({
			text = g_i18n:getText("ui_cge_preset_warning_remove"),
			callback = self.removePreset,
			target = self,
			args = {
				preset = preset
			}
		})
	else
		if #self.presets >= self.maxPresets then
			g_gui:showInfoDialog({
				text = g_i18n:getText("ui_cge_preset_warning_limit")
			})
		else
			g_gui:showTextInputDialog({
				text = g_i18n:getText("button_enterText"),
				maxCharacters = 20,
				callback = function (name, clickOk)
					if clickOk then
						if name == "" then
							name = g_i18n:getText("ui_cge_preset_unnamed")
						end

						for _, char in pairs({'<', '>', ':', '"', '/', '\\', '|', '?', '*'}) do
							name = string.gsub(name, char, "_")
						end

						if self:getPresetByName(name) == nil then
							self:addPreset(name, true)
						else
							g_gui:showInfoDialog({
								text = g_i18n:getText("ui_cge_preset_warning_exists")
							})
						end
					end
				end,
				confirmText = g_i18n:getText("button_save")
			})
		end
	end
end

function ColorGradingEditorScreen:getDefaultPreset()
	for i, preset in pairs(self.presets) do
		if preset.isDefault then
			return preset, i
		end
	end
end

function ColorGradingEditorScreen:getCustomPreset()
	for i, preset in pairs(self.presets) do
		if preset.isCustom then
			return preset, i
		end
	end
end

function ColorGradingEditorScreen:getPresetByName(name)
	for i, preset in pairs(self.presets) do
		if preset.isUser and preset.name == name then
			return preset, i
		end
	end
end

function ColorGradingEditorScreen:getCurrentPreset()
	return self.presets[self.currentPresetId]
end

function ColorGradingEditorScreen:clamp(value)
	return MathUtil.clamp(value, self.minValue, self.maxValue)
end

function ColorGradingEditorScreen:updatePresetTexts()
	local texts = {
		g_i18n:getText("setting_off"),
		g_i18n:getText("ui_cge_settings_presets_unsaved")
	}

	for _, preset in pairs(self.presets) do
		if preset.isUser then
			table.insert(texts, preset.name)
		end
	end

	self.multiPreset:setTexts(texts)

	local agsMultiColorGrading = self.colorGradingEditor.agsMultiColorGrading

	if agsMultiColorGrading ~= nil then
		agsMultiColorGrading:setTexts(texts)
	end
end

function ColorGradingEditorScreen:updateSaveButtonText()
	local text = g_i18n:getText("button_save")

	if self:getCurrentPreset().isUser then
		text = g_i18n:getText("button_delete")
	end

	self.defaultMenuButtonInfo[1].text = text
	self:assignMenuButtonInfo(self.defaultMenuButtonInfo)
end

function ColorGradingEditorScreen:updateSlidersValues()
	for _, page in pairs(self.pages) do
		page.itemList:reloadData()
	end
end

function ColorGradingEditorScreen:onSlidersUpdate(page, slider, value)
	local preset, id = self:getCustomPreset()

	if self.multiPreset.state ~= id then
		local currentPreset = self:getCurrentPreset()

		self.multiPreset:setState(id, false)

		local agsMultiColorGrading = self.colorGradingEditor.agsMultiColorGrading

		if agsMultiColorGrading ~= nil then
			agsMultiColorGrading:setState(id, false)
		end

		self.currentPresetId = id

		self:updateSaveButtonText()

		local tempDayFilename = preset.colorGrading.day.filename
		local tempNightFilename = preset.colorGrading.night.filename

		preset.colorGrading = table.copy(currentPreset.colorGrading, math.huge)
		preset.colorGrading.day.filename = tempDayFilename
		preset.colorGrading.night.filename = tempNightFilename
	end

	local forceSave = false

	local item = slider.parent
	local sectionIndex = item.sectionIndex
	local index = item.indexInSection

	local sections = preset.colorGrading[page.dayName]
	local groups = sections[page.sectionName]

	for i, groupName in pairs(self.groupNames) do
		if sectionIndex == i and groups[groupName] ~= nil then
			groups[groupName][index] = value
			forceSave = true

			break
		end
	end

	if sectionIndex == 5 and index == 1 then
		if page.sectionName == "shadows" then
			sections.shadowsMaxLuminance = value
			forceSave = true
		elseif page.sectionName == "highlights" then
			sections.highlightsMinLuminance = value
			forceSave = true
		end
	end

	if forceSave then
		self.saveTimer = self.saveDelay
		self.saveTimerEnabled = true
	end

	self:playSample(self.sliderSample)
end

function ColorGradingEditorScreen:addPreset(name, showDialog)
	local path = self.presetsDirectory .. "/" .. name

	createFolder(path)

	local dayFilename = path .. "/colorGradingDay.xml"
	local nightFilename = path .. "/colorGradingNight.xml"

	local preset = self:getCustomPreset()

	local newPreset = {
		name = name,
		isUser = true,
		colorGrading = table.copy(preset.colorGrading, math.huge
	)}

	newPreset.colorGrading.day.filename = dayFilename
	newPreset.colorGrading.night.filename = nightFilename

	if self:saveXMLFile(newPreset.colorGrading.day) and self:saveXMLFile(newPreset.colorGrading.night) then
		table.insert(self.presets, newPreset)

		table.sort(self.presets, function (a, b)
			return a.name:upper() < b.name:upper()
		end)

		local id = 1

		for i, preset in pairs(self.presets) do
			if preset == newPreset then
				id = i
				break
			end
		end

		self:updatePresetTexts()
		self.multiPreset:setState(id, true)

		if showDialog then
			g_gui:showInfoDialog({
				text = self.l10n:getText("ui_savingFinished")
			})
		end
	end
end

function ColorGradingEditorScreen:removePreset(yes, args)
	if yes and args.preset.isUser then
		if table.removeElement(self.presets, args.preset) then
			deleteFolder(self.presetsDirectory .. "/" .. args.preset.name)
			self:updatePresetTexts()

			local _, defaultPresetId = self:getDefaultPreset()

			self.multiPreset:setState(defaultPresetId, true)
		end
	end
end

function ColorGradingEditorScreen:setBlurApplied(blurApplied)
	if blurApplied then
		g_depthOfFieldManager:pushArea(self.background.absPosition[1], self.background.absPosition[2], self.background.absSize[1], self.background.absSize[2])
	else
		g_depthOfFieldManager:popArea()
	end

	self.blurApplied = blurApplied
end

function ColorGradingEditorScreen:loadSettingsFromXMLFile()
	local xmlFile = XMLFile.loadIfExists("settingsXML", self.settingsXMLFilename, "settings")

	if xmlFile ~= nil then
		local settings = self.settings

		settings.isCustomPreset = xmlFile:getBool("settings.isCustomPreset", settings.isCustomPreset)
		settings.presetName = xmlFile:getString("settings.presetName", settings.presetName)
		settings.showHelp = xmlFile:getBool("settings.showHelp", settings.showHelp)
		settings.bg = xmlFile:getInt("settings.bg", settings.bg)

		xmlFile:delete()
	end
end

function ColorGradingEditorScreen:saveSettingsToXMLFile()
	local settings = self.settings

	if settings.changed then
		local xmlFile = XMLFile.create("settingsXML", self.settingsXMLFilename, "settings")

		if xmlFile ~= nil then
			xmlFile:setBool("settings.isCustomPreset", settings.isCustomPreset)
			xmlFile:setString("settings.presetName", settings.presetName)
			xmlFile:setBool("settings.showHelp", settings.showHelp)
			xmlFile:setInt("settings.bg", settings.bg)

			xmlFile:save()
			xmlFile:delete()
		end

		settings.changed = false
	end
end

function ColorGradingEditorScreen:getSetting(name)
	return self.settings[name]
end

function ColorGradingEditorScreen:updateSetting(name, value)
	if self.settings[name] ~= nil and self.settings[name] ~= value then
		self.settings[name] = value
		self.settings.changed = true

		return true
	end

	return false
end

function ColorGradingEditorScreen:saveSettings()
	local preset = self:getCurrentPreset()

	self:updateSetting("isCustomPreset", preset.isCustom == true)
	self:updateSetting("presetName", preset.name)
	self:updateSetting("showHelp", self.showHelp)
	self:updateSetting("bg", self.bgState)

	self:saveSettingsToXMLFile()
end

function ColorGradingEditorScreen:updateDefaultPreset(dayFilename, nightFilename)
	local preset, id = self:getDefaultPreset()

	preset.colorGrading = self:loadColorGradingFromFiles(dayFilename, nightFilename)

	if id == self.currentPresetId then
		self:updateSlidersValues()
	end
end

function ColorGradingEditorScreen:updateColorGrading()
	setColorGradingSettings(self.defaultColorGrading, self.defaultColorGrading, 1)

	local lighting = g_currentMission.environment.lighting

	if lighting.colorGradingFileCurve == nil then
		lighting:updateCurves()
	end

	lighting:update(1, true)
end

function ColorGradingEditorScreen:lighting_update(lighting, superFunc, dt, force)
	local oldGetFunc = lighting.colorGradingFileCurve.get
	local isValid = g_currentMission.environment.baseLighting or lighting == self.colorGradingEditor:agsModGetCustomLighting()

	if isValid then
		local function newGetFunc(curve, time)
			local oldDay = lighting.colorGradingDay
			local oldNight = lighting.colorGradingNight

			local preset = self:getCurrentPreset()
			local newDay = preset.colorGrading.day.filename
			local newNight = preset.colorGrading.night.filename

			if preset.isDefault or not fileExists(newDay) or not fileExists(newNight) then
				if self.showCurrentGrading then
					print(string.format("Day: '%s'\nNight: '%s'", oldDay, oldNight))
				end

				return oldGetFunc(curve, time)
			else
				if self.showCurrentGrading then
					print(string.format("Day: '%s'\nNight: '%s'", newDay, newNight))
				end

				lighting.colorGradingDay = newDay
				lighting.colorGradingNight = newNight
			end

			local fileCurve = lighting:getColorGradingFileCurve()
			local first, second, alpha = fileCurve:get(time)

			lighting.colorGradingDay = oldDay
			lighting.colorGradingNight = oldNight

			return first, second, alpha
		end

		lighting.colorGradingFileCurve.get = newGetFunc
	end

	superFunc(lighting, dt, force)

	if isValid then
		lighting.colorGradingFileCurve.get = oldGetFunc
	end
end

function ColorGradingEditorScreen:consoleCommandShowCurrentGrading()
	self.showCurrentGrading = not self.showCurrentGrading

	return "Show current grading = " .. tostring(self.showCurrentGrading)
end